@@ -1,6 +1,6 @@
 from dataclasses import dataclass, field
 from pathlib import Path
-from typing import Callable, Dict, Optional, Tuple
+from typing import Callable, Dict, Optional, Tuple, List
 
 import numpy as np
 from catalyst.contrib.tools.tensorboard import SummaryWriter
@@ -11,40 +11,52 @@ from dnn_cool.task_flow import TaskFlow
 from dnn_cool.utils import any_value
 
 
-def publish_all(prefix, sample, key, writer, mapping, task_name):
-    if key in mapping:
-        publishers = mapping[key]
+def publish_all(prefix, sample, mapping_key, key, writer, mapping, task_name):
+    if mapping_key in mapping:
+        publishers = mapping[mapping_key]
         for publisher in publishers:
-            publisher(writer, sample, prefix, task_name)
+            publisher(writer, sample, prefix, task_name, key)
 
 
+def img(writer: SummaryWriter, sample: Tuple, prefix: str, task_name: str, key: str):
+    X, y = sample
+    writer.add_image(f'{prefix}_{task_name}_images', X[key])
+
+
+def text(writer: SummaryWriter, sample: Tuple, prefix: str, task_name: str, key: str):
+    X, y = sample
+    writer.add_text(f'{prefix}_{task_name}_text', X[key])
+
+
+def default_tensorboard_type_mapping():
+    return {
+        'img': [img],
+        'text': [text]
+    }
+
+
+@dataclass()
 class TensorboardConverter:
-    task_mapping = {}
-    col_mapping = {}
-    type_mapping = {}
-
-    def __init__(self):
-        self.type_mapping['img'] = [self.img]
+    col_mapping: Dict[str, List[Callable]] = field(default_factory=lambda: {})
+    type_mapping: Dict[str, List[Callable]] = field(default_factory=default_tensorboard_type_mapping)
+    col_to_type_mapping: Dict[str, str] = field(default_factory=lambda: {})
 
     def __call__(self, writer: SummaryWriter, sample: Tuple, prefix: str, task_name: str):
         if task_name == 'gt':
             return
         X, y = sample
         for key in X:
-            publish_all(prefix, sample, key, writer, self.col_mapping, task_name)
+            publish_all(prefix, sample, key, key, writer, self.col_mapping, task_name)
         for key in X:
-            publish_all(prefix, sample, key, writer, self.type_mapping, task_name)
-
-    def img(self, writer: SummaryWriter, sample: Tuple, prefix: str, task_name: str):
-        X, y = sample
-        writer.add_image(f'{prefix}_{task_name}_images', X['img'])
+            if key in self.col_to_type_mapping:
+                publish_all(prefix, sample, self.col_to_type_mapping[key], key, writer, self.type_mapping, task_name)
 
 
 @dataclass
 class TensorboardConverters:
     logdir: Path
     datasets: Dict[str, Dataset]
-    tensorboard_loggers: Callable = TensorboardConverter()
+    tensorboard_loggers: Callable = field(default_factory=TensorboardConverter)
     loggers: Dict[str, SummaryWriter] = field(default_factory=lambda: {})
     top_k: int = 10
 
