@@ -1,8 +1,7 @@
-import numpy as np
 import torch
-from catalyst.core import MultiMetricCallback
 from torch import nn
 
+from dnn_cool.dsl import IFeaturesDict, IOut, ICondition, IFlowTaskResult
 from dnn_cool.utils import any_value
 
 
@@ -22,7 +21,7 @@ class ReducedPerSample(nn.Module):
         return loss_results
 
 
-class LossFlowData:
+class LossFlowData(IFeaturesDict):
 
     def __init__(self, outputs, targets):
         self.outputs = outputs
@@ -33,12 +32,12 @@ class LossFlowData:
         return self
 
 
-class LossItems:
+class LossItems(IOut, IFlowTaskResult, ICondition):
 
     def __init__(self, loss_items):
         self.loss_items = loss_items
 
-    def __add__(self, other):
+    def __iadd__(self, other):
         return LossItems(self.loss_items + other.loss_items)
 
     # None of the methods below modify the state. They are here
@@ -193,12 +192,8 @@ class TaskFlowLoss(nn.Module):
                                                    loss.metric)
             callbacks.append(MetricCallback(f'loss_{path}', metric_decorator))
         for metric_name, metric_decorator in self.get_metrics():
-            metric = metric_decorator.metric
             full_name = f'{metric_name}_{metric_decorator.prefix}{metric_decorator.task_name}'
-            if metric.is_multi_metric():
-                callback = MultiMetricCallback(full_name, metric_decorator, metric.list_args())
-            else:
-                callback = MetricCallback(full_name, metric_decorator)
+            callback = MetricCallback(full_name, metric_decorator)
             callbacks.append(callback)
         return callbacks
 
